#!/usr/bin/env python

# (c) Simon Burton 2001
# this code is licensed GPL, see file LICENSE

# call with list of include file's full pathnames as arguments
# will create .py versions in current directory

import os,sys,string
import tempfile
import traceback
from types import *
from StringIO import *

ops=[
">>=", "<<=", "+=", "-=", "*=", "/=", "%=", "&=",
"^=", "|=", ">>", "<<", "++", "--", "->", "&&",
"||", "<=", ">=", "==", "!=", ";", "{", "}", ",",
":", "=", "(", ")", "[", "]", ".", "&", "!", "~",
"-", "+", "*", "/", "%", "<", ">", "^", "|", "?"]

def command(s):
  tmp=tempfile.mktemp()
  ret= os.system(s+" > "+tmp)
  if ret:
    sys.stderr.write( 
      "h2py: WARNING: "+repr(s)+" returned "+str(ret)+"\n" )
    return None
  file=open(tmp)
  return file

def output_header(file,env):
  outs="import struct\n"+\
   "def sizeof(type):\n"+\
   "  return struct.calcsize(type)\n\n"
  exec outs in env
  file.write(outs)
builtins=["sizeof"]

def tokenize(s):
  if not s:
    return []
  tok=string.split(s)
  #tok=[s]
  #print tok
  for op in ops:
    i=0
    while i < len(tok):
      _tok=tok[i]
      if (_tok in ops):
        i=i+1
        continue
      if string.count(_tok,op):
        j=string.index(_tok,op)
        __tok=[]
        if(j):
          __tok.append(_tok[0:j])
        __tok.append(op)
        if(_tok[j+len(op):]):
          __tok.append(_tok[j+len(op):])
        tok[i:i+1]=__tok
        i=i+2
      else:
        i=i+1
  return tok

def get_defines(pathname,macros):
  output=command("cpp "+pathname+" -dM ")
  if not output:
    return output
  for line in output.readlines():
    if line[:7]=="#define":
      #print line,
      assert line[-1]=='\n'
      line=line[7:-1]
      hasargs=string.count(string.split(line)[0],"(") or \
        string.split(line)[0][0]=="("
      toks=tokenize(line)
      args=[]
      i=1
      if hasargs:
        #print "#define ",line
        #print "#",string.join(toks)
        assert toks[i]=="("
        i=i+1
        while toks[i]!=")":
          if toks[i]!=",":
            args.append(toks[i])
          i=i+1
        i=i+1
        #print "#args ",args
        #print 
      if toks[1:]:
        macro=Macro(toks[0],args,toks[i:])
      else:
        macro=Macro(toks[0],args,["None"])
      macros[toks[0]]=macro
      #print macro.py()
      #print "         ",macro.deps
    else:
      sys.stderr.write("ignoring : "+line);
  return macros

# we send curly brackets to square brackets,
# && to and, || to or
mapping={"{":"[","}":"]","&&":"and","||":"or"}

# we send (some) types to format encodings
ctype={
"signed char":'b', 
"unsigned char":'B', 
"char":'c', 
"short":'h',
"unsigned short":'H',
"int":'i',
"unsigned int":'I',
"long":'l',
"unsigned long":'L',
"float":'f',
"double":'d'}

class Macro:
  def __init__(self,name,args,sub):
    self.name=name
    self.args=args # list
    self.sub=sub # tokenized
    self.deps=[]
    i=0
    while i < len(sub):
      for key in mapping.keys():
        if sub[i]==key:
          sub[i]=mapping[key]
          break
      else:
        s=sub[i]
        if s=="None":
          pass
        elif s[0] in string.digits:
          # a number
          if s[-1:]=='U':
            # unsigned, forget about it
            sub[i]=sub[i][:-1]
        elif s[0]=="'":
          # char literal
          sub[i]="ord("+sub[i]+")"
        elif not s in args and s[0] in string.letters+"_":
          if s in builtins:
            pass
          else:
            for ckey in map(string.split,ctype.keys()):
              if sub[i:i+len(ckey)]==ckey:
                #print " mapping ",sub[i:],
                sub[i:i+len(ckey)]=["'"+ctype[string.join(ckey)]+"'"]
                #print " to ",sub[i]
                #sys.stderr.write(" got "+string.join(ckey)+"\n")
                #i=i+len(ckey)-1
                break
            else:
              self.deps.append(s)
      i=i+1 
  def py(self):
    if self.args:
      s=string.join(self.args,",")
      return "def "+self.name+"("+s+")"+":"+\
        "\n  return "+string.join(self.sub)
    else:
      return self.name+"="+string.join(self.sub)
  def sub(self,args):
    assert len(args)==len(self.args)
    sub=[]
    for s in self.sub:
      if s in args:
        pass

def output_py(
  file,macros,_macros,__macros,name,env):
  " macros: parsed macros " 
  " _macros: outputed macros "
  " __macros: locked; can't output these "
  " macro: macro name to output "
  if _macros.has_key(name):
    return
  macro=macros[name]
  for d in macro.deps:
    if __macros.has_key(d) or d==name:
      file.write("#recursive definition of "+repr(name)+"\n")
      return 1
    if not macros.has_key(d):
      file.write("#"+repr(d)+" undefined in "+repr(name)+"\n")
      return 1
    __macros[d]=macros[d]
    er=output_py(
      file,macros,_macros,__macros,d,env)
    del __macros[d]
    if er:
      file.write("#"+repr(name)+": failed dependancy:"+repr(d)+"\n")
      return 1
  else:
    outs=macro.py()
    try:
      _outs=string.replace(outs,"\n","")
      #exec(outs,global_env,local_env)
      exec(_outs,env)
    except SyntaxError:
      #file.write("#"+_outs+"\n")
      sfile=StringIO()
      traceback.print_exc(None,sfile)
      s=string.replace(sfile.getvalue(),"\n","\n#")
      sfile.close()
      s="#"+s[:-1]
      file.write(s)
      file.write("#skipping "+repr(macro.name)+"\n")
      return 1
    except Exception:
      file.write("#"+outs+"\n")
      sfile=StringIO()
      traceback.print_exc(None,sfile)
      s=string.replace(sfile.getvalue(),"\n","\n#")
      sfile.close()
      s="#"+s[:-1]
      file.write(s)
      file.write("#skipping "+repr(macro.name)+"\n")
      #file.write("#namespace dump:\n")
      #for k in env.keys():
      #  file.write( "#"+k +"\n")
      return 1
    else:
      pass
    # success!
    assert not _macros.has_key(name)
    file.write( outs+"\n")
    _macros[name]=macro
  return 0 # ok

def send(path):
  sys.stderr.write("h2py: reading file : "+path+"\n")
  macros=get_defines(path,{})
  if not macros:
    return
  kern=string.split(os.path.split(path)[1],".")[0]
  file=open(kern+".py","w+")
  env={}
  sys.stderr.write("h2py: writing file : "+kern+".py\n")
  output_header(file,env)
  keys=macros.keys()
  keys.sort()
  _macros={}
  __macros={}
  for k in keys:
    output_py(
      file,macros,_macros,__macros,k,env)
  file.close()

if __name__=="__main__":
  for path in sys.argv[1:]:
    send(path)
  if not sys.argv[1:]:
    print " call with list of include ",
    print "file's full pathnames as arguments."
    print "Will create .py versions in current directory"

